#include "ngap_types.hh"

#include "NGAP_TestSystem.hh"

#include "ngap_layer_factory.hh"
//#include "geonetworking_layer_factory.hh"

#include "registration.hh"

#include "loggers.hh"

ngap_layer::ngap_layer(const std::string &p_type, const std::string &param) : t_layer<LibNGAP__Interface::NGAPPort>(p_type), _params(), _codec() {
  loggers::get_instance().log(">>> ngap_layer::ngap_layer: %s, %s", to_string().c_str(), param.c_str());
  // Setup parameters
  params::convert(_params, param);
//  _params.insert(std::make_pair<std::string, std::string>("its_aid", "36"));     // ETSI TS 102 965 V1.2.1 (2015-06)
//  _params.insert(std::make_pair<std::string, std::string>("payload_type", "2")); // CA message id - See ETSI TS 102 894

  // Register this object for AdapterControlPort
  loggers::get_instance().log("ngap_layer::ngap_layer: Register %s/%p", p_type.c_str(), this);
  registration<ngap_layer>::get_instance().add_item(p_type, this);
}

void ngap_layer::sendMsg(const NGAP__PDU__Descriptions::NGAP__PDU& p, params &params) {
  loggers::get_instance().log_msg(">>> ngap_layer::sendMsg: ", p);

  // Encode NGAP PDU
  OCTETSTRING data = int2oct(0,1);
  //OCTETSTRING data;
  //  loggers::get_instance().warning("ngap_layer::sendMsg: Try to encode...");

  //if (_codec){   
   if (_codec.encode(p, data) < 0) {
      loggers::get_instance().warning("ngap_layer::sendMsg: Encoding failure");
      data=int2oct(0,1);
      //return;
   }
   loggers::get_instance().log_msg(">>> ngap_layer::sendMsg . encoded: ", data);
  //}else{
//	        loggers::get_instance().warning("ngap_layer::sendMsg: codec not set");
  //}
  send_data(data, _params);
}

void ngap_layer::send_data(OCTETSTRING &data, params &p_params) {
  loggers::get_instance().log_msg(">>> ngap_layer::send_data: ", data);

  send_to_all_layers(data, p_params);
}

void ngap_layer::receive_data(OCTETSTRING &data, params &p_params) {
  loggers::get_instance().log_msg(">>> ngap_layer::receive_data: ", data);

  // Sanity check
  //if (*(static_cast<const unsigned char *>(data) + 1) != 0x02) { // Check that received packet has CA message id - See ETSI TS 102 894
  //  // Not a NGAP message, discard it
  //  loggers::get_instance().warning("ngap_layer::receive_data: Wrong message id: 0x%02x", *(static_cast<const unsigned char *>(data) + 1));
  //  return;
  //}

  // Decode the payload
  //params &params = p_params;//static_cast<params&>(p_params);

  NGAP__PDU__Descriptions::NGAP__PDU p;
  _codec.decode(data, p, &p_params);
  /*if (!p.msgIn().is_bound()) {
    // Discard it
    return;
  } // else, continue

  // Process lower layer data
  // recvTime
  params::const_iterator it = params.find(params::timestamp);
  if (it != params.cend()) {
    p.recvTime().set_long_long_val(std::stoll(it->second));
  } else {
    p.recvTime().set_to_omit();
  }
  // gnNextHeader
  it = params.find(params_its::gn_next_header);
  if (it != params.cend()) {
    p.gnNextHeader() = std::stoi(it->second);
  } else {
    p.gnNextHeader().set_to_omit();
  }
  // gnHeaderType
  it = params.find(params_its::gn_header_type);
  if (it != params.cend()) {
    p.gnHeaderType() = std::stoi(it->second);
  } else {
    p.gnHeaderType().set_to_omit();
  }
  // gnHeaderSubtype
  it = params.find(params_its::gn_header_sub_type);
  if (it != params.cend()) {
    p.gnHeaderSubtype() = std::stoi(it->second);
  } else {
    p.gnHeaderSubtype().set_to_omit();
  }
  // gnLifetime
  it = params.find(params_its::gn_lifetime);
  if (it != params.cend()) {
    p.gnLifetime() = std::stoi(it->second);
  } else {
    p.gnLifetime().set_to_omit();
  }
  // gnTrafficClass
  it = params.find(params_its::gn_traffic_class);
  if (it != params.cend()) {
    p.gnTrafficClass() = std::stoi(it->second);
  } else {
    p.gnTrafficClass().set_to_omit();
  }
  // btpDestinationPort
  it = params.find(params_its::btp_destination_port);
  if (it != params.cend()) {
    p.btpDestinationPort() = std::stoi(it->second);
  } else {
    p.btpDestinationPort().set_to_omit();
  }
  // btpInfo
  it = params.find(params_its::btp_info);
  if (it != params.cend()) {
    p.btpInfo() = std::stoi(it->second);
  } else {
    p.btpInfo().set_to_omit();
  }
  // ssp
  it = params.find(params_its::ssp);
  if (it != params.cend()) {
    loggers::get_instance().log("ngap_layer::receive_data: ssp=%s", it->second.c_str());
    p.ssp() = oct2bit(str2oct(CHARSTRING(it->second.c_str())));
  } else {
    p.ssp().set_to_omit();
  }
  // its_aid
  it = params.find(params_its::its_aid);
  if (it != params.cend()) {
    p.its__aid() = std::stoi(it->second);
  } else {
    p.its__aid().set_to_omit();
  }
*/
  // Pass it to the ports if any
  to_all_upper_ports(p, p_params);

  loggers::get_instance().log_msg(">>> ngap_layer::receive_data:TO UPPER PORT ", p);
}

/*int ngap_layer::enable_secured_mode(const std::string &p_certificate_id, const boolean p_enforce_security) {
  loggers::get_instance().log(">>> ngap_layer::enable_secured_mode: '%s' - %x", p_certificate_id.c_str(), p_enforce_security);

  geonetworking_layer *p = registration<geonetworking_layer>::get_instance().get_item(std::string("GN"));
  if (p == NULL) {
    return -1;
  }
  
  loggers::get_instance().log("ngap_layer::enable_secured_mode: Got GN layer %p", p);
  return p->enable_secured_mode(p_certificate_id, p_enforce_security);
}

int ngap_layer::disable_secured_mode() {
  loggers::get_instance().log(">>> ngap_layer::disable_secured_mode");

  geonetworking_layer *p = registration<geonetworking_layer>::get_instance().get_item(std::string("GN"));
  if (p == NULL) {
    return -1;
  }
  
  loggers::get_instance().log("ngap_layer::disable_secured_mode: Got GN layer %p", p);
  return p->disable_secured_mode();
}*/

ngap_layer_factory ngap_layer_factory::_f;
